/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.DataExchange;

import java.io.IOException;
import java.util.HashMap;
import java.util.Properties;
import java.util.Vector;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import com.sos.JSHelper.Basics.JSJobUtilities;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Options.SOSOptionJadeOperation.enuJadeOperations;
import com.sos.VirtualFileSystem.DataElements.SOSFileList;
import com.sos.VirtualFileSystem.DataElements.SOSFileListEntry;
import com.sos.VirtualFileSystem.FTP.SOSFTPOptions;
import com.sos.VirtualFileSystem.Factory.VFSFactory;
import com.sos.VirtualFileSystem.Interfaces.ISOSVFSHandler;
import com.sos.VirtualFileSystem.Interfaces.ISOSVfsFileTransfer;
import com.sos.VirtualFileSystem.Interfaces.ISOSVirtualFile;
import com.sos.i18n.I18NBase;
import com.sos.i18n.annotation.I18NMessage;
import com.sos.i18n.annotation.I18NMessages;
import com.sos.i18n.annotation.I18NResourceBundle;

@I18NResourceBundle(baseName = "SOSDataExchange", defaultLocale = "en")
public class SOSDataExchangeEngine extends I18NBase implements Runnable, JSJobUtilities {
	private static final String	conKeyWordLAST_ERROR			= "last_error";
	private static final String	conKeywordSTATE					= "state";
	private static final String	conKeywordSUCCESSFUL_TRANSFERS	= "successful_transfers";
	private static final String	conKeywordFAILED_TRANSFERS		= "failed_transfers";
	private static final String	conKeywordSTATUS				= "status";
	private final String		conClassName					= "SOSDataExchangeEngine";
	public final String			conSVNVersion					= "$Id: SOSDataExchangeEngine.java 16361 2012-01-27 12:27:35Z kb $";

	final String				SOSJADE_D_0200					= "SOSJADE_D_0200";													// "source directory/file: "
	final String				SOSJADE_E_0200					= "SOSJADE_E_0200";													// "Problems creating/connecting DataSourceClient";

	final String				SOSJADE_E_0101					= "SOSJADE_E_0101";													// "data transfer ended with error %1$s";
	final String				SOSJADE_I_0115					= "SOSJADE_I_0115";													// "getList is specified. no transfer will be done.";

	final String				SOSJADE_I_0102					= "SOSJADE_I_0102";													// "%1$d files skipped due to zero byte constraint";
	final String				SOSJADE_I_0101					= "SOSJADE_I_0101";													// "%1$d files transferred";
	final String				SOSJADE_I_0100					= "SOSJADE_I_0100";													// "one file transferred";
	final String				SOSJADE_I_0104					= "SOSJADE_I_0104";													// "zero byte file(s) found";

	private Logger				logger							= Logger.getLogger(SOSDataExchangeEngine.class);
	final String				strLoggerName					= "JadeReportLog";
	private Logger				objJadeReportLogger				= Logger.getLogger(strLoggerName);

	final String				SOSJADE_E_0100					= "SOSJADE_E_0100";													// "No file name found which match the regular expression criteria '1$s'";

	private ISOSVFSHandler		objVfs4Target					= null;
	private ISOSVFSHandler		objVfs4Source					= null;
	public ISOSVfsFileTransfer	objDataTargetClient				= null;
	public ISOSVfsFileTransfer	objDataSourceClient				= null;
	private SOSFTPOptions		objOptions						= null;
	private SOSFileList			objSourceFileList				= null;
	private boolean				jump							= false;																// TODO
																																		// temporr
	private boolean				makeDirs						= false;
	/** This parameter specifies whether zero byte files should be transferred and processed by subsequent jobs. The following settings are available */
	private boolean				zeroByteFiles					= true;
	private boolean				zeroByteFilesStrict				= false;
	private boolean				zeroByteFilesRelaxed			= false;
	/** notification by e-mail in case of transfer of empty files. */
	private String				fileNotificationTo				= "";
	private String				fileNotificationCC				= "";
	private String				fileNotificationBCC				= "";
	private String				fileNotificationSubject			= "";
	private String				fileNotificationBody			= "";
	private String				fileZeroByteNotificationTo		= "";
	private String				fileZeroByteNotificationCC		= "";
	private String				fileZeroByteNotificationBCC		= "";
	private String				fileZeroByteNotificationSubject	= "";
	private String				fileZeroByteNotificationBody	= "";
	private JSJobUtilities		objJSJobUtilities				= this;
	// private boolean sameConnection = false;
	// private int count = 0;
	private int					zeroByteCount					= 0;																	// temporr
	// private boolean isFilePath = false;
	// private String state = "";
	/**
	 * Resource filenames for operation = install
	 */
	@SuppressWarnings("unused")
	private boolean				testmode						= false;
	@I18NMessages(value = { @I18NMessage("%1$s: Exception raised, reason is: %2$s"), //
			@I18NMessage(value = "%1$s: Exception raised, reason is: %2$s", locale = "en_UK", //
			explanation = "%1$s: Exception raised, cause: %2$s" //
			), //
			@I18NMessage(value = "%1$s: Ausnahmefehler aufgetreten, Ursache ist vermutlich: %2$s", locale = "de", //
			explanation = "%1$s: Eine Java-Exception mit dem Grund: %2$s ist aufgetreten." //
			) //
	}, msgnum = "SOS-E-1001", msgurl = "")
	/*!
	 * \var EXCEPTION_RAISED
	 * \brief %1$s: Exception raised, cause: %2$s
	 */
	final String				EXCEPTION_RAISED				= conClassName + ".EXCEPTION_RAISED";
	@I18NMessages(value = { @I18NMessage("could not complete transaction, cause: {0}"), //
			@I18NMessage(value = "could not complete transaction, cause: {0}", locale = "en_UK", //
			explanation = "could not complete transaction, cause: {0}" //
			), //
			@I18NMessage(value = "Transaktion wurde fehlerhaft abgebrochen, Ursache: {0}", locale = "de", //
			explanation = "Die Transaktion wurde abgebrochen und alle Aktionen rckgngig gemacht." //
			) //
	}, msgnum = "SOSDX-E-0002", msgurl = "")
	/*!
	 * \var TRANSACTION_ABORTED
	 * \brief could not complete transaction, cause: %1$s
	 */
	final String				TRANSACTION_ABORTED				= "SOSDataExchangeEngine.TRANSACTION_ABORTED";

	public SOSDataExchangeEngine() throws Exception {
		super("SOSDataExchange");
		init();
	}

	private void init() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::init";

		logger.info(conClassName + " --- " + conSVNVersion);

	} // private void init

	/**
	 * @param settingsFile
	 * @param settingsSection
	 * @param logger
	 * @param arguments_
	 */
	public SOSDataExchangeEngine(Properties pobjProperties) throws Exception {
		super("SOSDataExchange");
		this.Options();
		// TODO Properties in die OptionsClasse weiterreichen
		// objOptions.setAllOptions(pobjProperties);
	}

	public SOSDataExchangeEngine(SOSFTPOptions pobjOptions) throws Exception {
		super("SOSDataExchange");
		this.objOptions = pobjOptions;
	}

	public SOSDataExchangeEngine(HashMap<String, String> pobjJSSettings) throws Exception {
		super("SOSDataExchange");
		this.Options();
		objOptions.setAllOptions(pobjJSSettings);
	}

	@SuppressWarnings("unused")
	private ISOSVFSHandler getVFS() throws Exception {
		if (objVfs4Target == null) {
			objVfs4Target = VFSFactory.getHandler(objOptions.getDataTargetType());
		}
		return objVfs4Target;
	}

	public SOSFTPOptions Options() {
		if (objOptions == null) {
			objOptions = new SOSFTPOptions();
		}
		return objOptions;
	}

	@Override
	public void run() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Run";
		try {
			this.Execute();
		}
		catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace(System.err);
		}
	}

	public boolean Execute() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::execute";

		VFSFactory.setParentLogger(strLoggerName);
		if (objOptions.verbose.value() <= 0) {
			Logger.getRootLogger().setLevel(Level.INFO);
		}
		else {
			Logger.getRootLogger().setLevel(Level.DEBUG);
		}
		// logger.debug(this.conSVNVersion);
		objOptions.getTextProperties().put("version", conSVNVersion);

		objOptions.log_filename.setLogger(objJadeReportLogger);
		String strT = Messages.getMsg("SOSJADE_T_0010"); // LogFile Header
		objJadeReportLogger.info(objOptions.replaceVars(strT));

		boolean flgOK = this.transfer();

		strT = Messages.getMsg("SOSJADE_T_0011"); // LogFile Footer
		objJadeReportLogger.info(objOptions.replaceVars(strT));
		return flgOK;
	}

	public ISOSVfsFileTransfer DataSourceClient() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::DataSourceClient";

		if (objDataSourceClient == null) {
			try {
				objVfs4Source = VFSFactory.getHandler(objOptions.getDataSourceType());
				objVfs4Source.Connect(objOptions.getConnectionOptions().Source());
				objVfs4Source.Authenticate(objOptions.getConnectionOptions().Source());
				objVfs4Source.setSource();
				objVfs4Source.Options(objOptions);
			}
			catch (Exception e) {
				throw new JobSchedulerException(Messages.getMsg(SOSJADE_E_0200), e);
			}
		}

		return objDataSourceClient;
	} // private ISOSVfsFileTransfer DataSourceClient

	/**
	 * 
	 * \brief DataTargetClient
	 * Returns an instance of the TargetClient
	 * 
	 * \details
	 *
	 * \return ISOSVfsFileTransfer
	 *
	 * @return
	 * @throws Exception
	 */
	public ISOSVfsFileTransfer DataTargetClient() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::DataTarget";
		if (objDataTargetClient == null) {
			objOptions.CheckMandatory();
			objVfs4Target = VFSFactory.getHandler(objOptions.getDataTargetType());
			objVfs4Target.Connect(objOptions.getConnectionOptions().Target());
			objVfs4Target.Authenticate(objOptions.getConnectionOptions().Target());
			objDataTargetClient = (ISOSVfsFileTransfer) objVfs4Target;
			objVfs4Target.setTarget();
			objVfs4Target.Options(objOptions);
		}
		return objDataTargetClient;
	} // private ISOSVfsFileTransfer DataTarget

	/**
	 * Send files by  .... from source to a target  
	 * 
	 * @return boolean
	 * @throws Exception
	 */
	public boolean transfer() throws Exception {
		boolean flgReturnCode = false;
		try { // to connect, authenticate and execute commands
			Options().CheckMandatory();
			setTextProperties();

			// logger.debug(Options().toString());
			objSourceFileList = new SOSFileList(objVfs4Target);
			objSourceFileList.Options(objOptions);
			objSourceFileList.StartTransaction();
			DataSourceClient();
			if (objOptions.NeedTargetClient() == true) {
				DataTargetClient();
				objSourceFileList.objDataTargetClient = objDataTargetClient;
				if (objOptions.passive_mode.value()) {
					objDataTargetClient.passive();
				}
				objDataTargetClient.TransferMode(objOptions.transfer_mode);
				objDataTargetClient.ControlEncoding(objOptions.ControlEncoding.Value());
				if (objOptions.PreFtpCommands.IsNotEmpty()) {
					String[] strA = objOptions.PreFtpCommands.Value().split(";");
					for (String strCmd : strA) {
						objDataTargetClient.getHandler().ExecuteCommand(strCmd);
					}
				}
				makeDirs();
			}

			try {
				objDataSourceClient = (ISOSVfsFileTransfer) objVfs4Source;
				objSourceFileList.objDataSourceClient = objDataSourceClient;
				if (objOptions.passive_mode.value()) {
					objDataSourceClient.passive();
				}

				objDataSourceClient.TransferMode(objOptions.transfer_mode);
				objDataSourceClient.ControlEncoding(objOptions.ControlEncoding.Value());

				String strSourceDir = objOptions.SourceDir.Value();
				String strRemoteDir = objOptions.TargetDir.Value();
				boolean flgRecurseFolders = objOptions.recursive.value();
				if (objOptions.OneOrMoreSingleFilesSpecified()) {
					objSourceFileList.add(getSingleFileNames(), strSourceDir);
				}
				else {
					String fileSpec = objOptions.file_spec.Value();
					objDataSourceClient.changeWorkingDirectory(strSourceDir);
					ISOSVirtualFile objLocFile = objDataSourceClient.getFileHandle(strSourceDir);
					String strM = "";
					if (objOptions.NeedTargetClient() == true) {
						strM = "source directory/file: " + strSourceDir + ", target directory: " + strRemoteDir + ", file regexp: " + fileSpec;
					}
					else {
						strM = Messages.getMsg(SOSJADE_D_0200, strSourceDir, fileSpec);
					}
					logger.debug(strM);
					if (objLocFile.isDirectory() == true) {
						String[] strFileList = objDataSourceClient.getFilelist(strSourceDir, fileSpec, 0, flgRecurseFolders);
						objSourceFileList.add(strFileList, strSourceDir);
					}
					else {
						objSourceFileList.add(strSourceDir);
					}
				}
				// TODO implement polling for files

				// TODO in Options setzen anhand der Values in der Liste.
				if (zeroByteFiles == false) {
					// TODO in die SOSFileList einbauen (SkipZeroByteFiles)
					for (SOSFileListEntry objEntry : objSourceFileList.List()) {
						String strSourceFileName = objEntry.SourceFileName();
						ISOSVirtualFile objVF = objDataSourceClient.getFileHandle(strSourceFileName);
						if (objVF.FileExists()) {
							if (objVF.isEmptyFile()) {
								objEntry.setTransferSkipped();
								if (objOptions.remove_files.value()) {
									objVF.delete();
									objEntry.zeroByteCount++;
								}
							}
						}
						else {
							// TODO Datei (nicht mehr) da? Fehler auslsen, weil in Liste enthalten.
						}
					}
				} // (zeroByteFiles == false)
				try {
					if (objOptions.operation.value() == enuJadeOperations.getlist) {
						String strM = Messages.getMsg(SOSJADE_I_0115);
						logger.info(strM);
						objJadeReportLogger.info(strM);
					}
					else {
						sendFiles(objSourceFileList);
					}

					objDataSourceClient.close();
					if (objOptions.NeedTargetClient() == true) {
						objDataTargetClient.close();
					}

					objSourceFileList.renameAtomicTransferFiles();
					objSourceFileList.DeleteSourceFiles();

					objSourceFileList.EndTransaction();
				}
				catch (JobSchedulerException e) {
					e.printStackTrace(System.err);
					String strM = getMsg(TRANSACTION_ABORTED, e.toString());
					if (objOptions.transactional.value()) {
						logger.error(strM, e);
					}
					else {
						logger.error(strM, e);
					}
					objJadeReportLogger.info(strM);
					objSourceFileList.Rollback();
					throw new JobSchedulerException(strM, e);
				}
				finally {
				}

				objSourceFileList.sendMails();
				// TODO in die Klasse SOSFileList oder SOSFtpCommand
				flgReturnCode = printState(flgReturnCode);
				// }
				setTextProperties();
				return flgReturnCode;
			}
			catch (Exception e) {
				setTextProperties();
				objOptions.getTextProperties().put(conKeywordSTATUS, Messages.getMsg("SOSJADE_T_0013"));
				flgReturnCode = false;
				e.printStackTrace(System.err);
				String strM = Messages.getMsg(SOSJADE_E_0101, e.getLocalizedMessage());
				objJadeReportLogger.info(strM);
				throw (new JobSchedulerException(strM));
			}
			finally {
			}
		}
		catch (Exception e) {
			e.printStackTrace(System.err);
			// if (sosString.parseToString(logger.getWarning()).length() > 0 || sosString.parseToString(logger.getError()).length() > 0) {
			// //TODO in die Order/Job-Parameter
			// getArguments().put("last_error", logger.getWarning() + " " + logger.getError());
			// }
			String strM = Messages.getMsg(SOSJADE_E_0101, e.getLocalizedMessage());
			objJadeReportLogger.info(strM);
			throw (new JobSchedulerException(strM));
		}
	}

	private void setTextProperties() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setTextProperties";

		// TODO das mu beim JobScheduler-Job in die Parameter zurck, aber nur da
		// siehe hierzu das Interface ...

		if (objSourceFileList != null) {
			objOptions.getTextProperties().put(conKeywordSUCCESSFUL_TRANSFERS, String.valueOf(objSourceFileList.SuccessfulTransfers()));
			objOptions.getTextProperties().put(conKeywordFAILED_TRANSFERS, String.valueOf(objSourceFileList.FailedTransfers()));
		}
		// return the number of transferred files
		objOptions.getTextProperties().put(conKeywordSTATUS, Messages.getMsg("SOSJADE_T_0012"));
		objOptions.getTextProperties().put(conKeyWordLAST_ERROR, "");

	} // private void setTextProperties

	/**
	 * 
	 * \brief Logout
	 * 
	 * \details
	 *
	 * \return void
	 *
	 */
	public void Logout() {
		try {
			if (objDataTargetClient != null) {
				objDataTargetClient.logout();
				objDataTargetClient.disconnect();
				objDataTargetClient = null;
			}
			if (objDataSourceClient != null) {
				objDataSourceClient.logout();
				objDataSourceClient.disconnect();
				objDataSourceClient = null;
			}
		}
		catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Verzeichnis generieren
	 * @throws Exception
	 */
	private void makeDirs() {
		try {
			boolean cd = true;
			// TODO make_dirs ist nicht dokumentiert.
			// boolean flgMakeDirs = objOptions.make_dirs.Value();
			boolean flgMakeDirs = makeDirs;
			String strTargetDir = objOptions.TargetDir.Value();
			if (flgMakeDirs) {
				if (objDataTargetClient.changeWorkingDirectory(strTargetDir)) {
					cd = true;
				}
				else {
					objDataTargetClient.mkdir(strTargetDir);
					cd = objDataTargetClient.changeWorkingDirectory(strTargetDir);
				}
			}
			else
				if (strTargetDir != null && strTargetDir.length() > 0) {
					cd = objDataTargetClient.changeWorkingDirectory(strTargetDir);
				}
			// TODO alternative_remote_dir, wozu und wie gehen wir damit um?
			if (cd == false && objOptions.alternative_remote_dir.IsNotEmpty()) {// alternative Parameter
				String alternativeRemoteDir = objOptions.alternative_remote_dir.Value();
				logger.debug("..try with alternative parameter [remoteDir=" + alternativeRemoteDir + "]");
				cd = objDataTargetClient.changeWorkingDirectory(alternativeRemoteDir);
				objOptions.TargetDir.Value(alternativeRemoteDir);
			}
		}
		catch (Exception e) {
			throw new JobSchedulerException("..error in makeDirs, cause: " + e);
		}
	} // private void makeDirs()

	// TODO prfen, ob eine Verlagerung in SOSFileList die bessere Lsung ist. Stichwort: Multithreading
	private void sendFiles(final SOSFileList pobjFileList) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::sendFiles";
		int intMaxParallelTransfers = 0;
		if (objOptions.ConcurrentTransfer.value() == true) {
			intMaxParallelTransfers = objOptions.MaxConcurrentTransfers.value();
		}
		Vector<Thread> objThreads = new Vector<Thread>();
		try {
			/**
			 * if "sendTransferHistory" is activated due to not thread-safe xml-parser
			 * the parallel transfer is omitted.
			 */
			if (intMaxParallelTransfers <= 0 || objOptions.SendTransferHistory.value() == true) {
				for (SOSFileListEntry objSourceFile : pobjFileList.List()) {
					objSourceFile.Options(objOptions);
					objSourceFile.run();
				}
			}
			else {
				int intThreadsRunning = 0;
				for (SOSFileListEntry objListItem : pobjFileList.List()) {
					objListItem.Options(objOptions);
					objListItem.setDataSourceClient(null); // force a new connection
					objListItem.setDataTargetClient(null);
					Thread objT = new Thread(objListItem);
					objT.start();
					synchronized (objThreads) {
						objThreads.add(objT);
					}
					boolean flgThreadsActive = true;
					intThreadsRunning = 0;
					while (flgThreadsActive) {
						flgThreadsActive = false;
						synchronized (objThreads) {
							for (Thread thread : objThreads) {
								if (thread.isAlive()) {
									flgThreadsActive = true;
									intThreadsRunning++;
								}
							}
						}
						try {
							if (flgThreadsActive == true && intThreadsRunning >= intMaxParallelTransfers) {
								Thread.sleep(500); // TODO this should be an option
							}
							if (intThreadsRunning < intMaxParallelTransfers) {
								break;
							}
						}
						catch (InterruptedException e) {
							e.printStackTrace();
							throw new JobSchedulerException("thread processing interrupted", e);
						}
						catch (Exception e) {
							e.printStackTrace();
							throw new JobSchedulerException("thread processing interrupted", e);
						}
					}
				}
			}
		}
		catch (JobSchedulerException e) {
			throw e;
		}
	}

	/**
	 * Statt ein Verzeichnis knnen ein oder mehrere - mit semikolen getrennt - dateien zum transferieren angegeben werden
	 * @return
	 */
	private String[] getSingleFileNames() {
		final String conMethodName = conClassName + "::getSingleFileNames";
		String filePath = objOptions.file_path.Value();
		// String strPS = System.getProperty("file.separator");
		// String strPS = "/"; // standard on FTP/SFTP
		logger.debug("single file(s) specified : " + filePath);
		Vector<String> filelist = new Vector<String>();
		try {
			String localDir = objOptions.SourceDir.Value().trim();
			// TODO separator-char variable as Option
			String[] split = filePath.split(";");
			for (String strT : split) {
				strT = strT.trim();
				if (strT.length() > 0) {
					if (localDir.trim().length() > 0) {
						if (isAPathName(strT) == false) {
							/**
							 * Problem with folders, when pgs run on Windows, but has to
							 * create a path for unix-systems.
							 */
							// strT = new File(localDir, strT).getAbsolutePath();
							strT = AddFileSeparator(localDir) + strT;
						}
					}
					filelist.add(strT);
				}
			}
		}
		catch (Exception e) {
			String strM = String.format("error in  %1$s", conMethodName);
			logger.error(strM + e);
			throw new JobSchedulerException(strM, e);
		}
		String[] strA = { "" };
		if (filelist.size() > 0) {
			strA = (String[]) filelist.toArray(new String[filelist.size()]);
		}
		return strA;
	}

	private boolean isAPathName(final String pstrPathName) {
		boolean flgOK = (pstrPathName.contains("\\") == true || pstrPathName.contains("/") == true);
		return flgOK;
	}

	private String AddFileSeparator(final String pstrPathName) {
		String strT = pstrPathName;
		int intL = strT.length();
		if (intL > 0) {
			String strL = strT.substring(intL - 1, intL);
			if (strL.equals("\\") || strL.equals("/")) {
			}
			else {
				// strT += File.separator;
				strT += "/";
			}
		}
		return strT;
	}

	/**
	 * Zustand ausgeben. Erst ab log_level 1
	 * @param rc
	 * @return
	 * @throws Exception
	 */
	private boolean printState(boolean rc) throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::printState";
		String state = "processing successful ended";

		try {
			int intNoOfFilesTransferred = (int) objSourceFileList.SuccessfulTransfers();
			String strMsg = Messages.getMsg(SOSJADE_E_0100, objOptions.file_spec.Value());
			switch (intNoOfFilesTransferred) {
				case 0:
					if (zeroByteCount > 0 && zeroByteFilesRelaxed) {
						state = strMsg;
					}
					else
						if (zeroByteCount > 0 && zeroByteFilesStrict) {
							throw new JobSchedulerException(Messages.getMsg(SOSJADE_I_0104));
						}
						else {
							if (objOptions.force_files.value() == true) {
								if (objSourceFileList.List().size() <= 0) {
									objJadeReportLogger.info(strMsg);
									throw new JobSchedulerException(strMsg);
								}
							}
							else {
								state = strMsg;
							}
						}
					rc = (objOptions.force_files.value() == false ? true : !zeroByteFilesRelaxed);
					break;
				case 1:
					state = Messages.getMsg(SOSJADE_I_0100);
					rc = true;
					break;
				default:
					state = Messages.getMsg(SOSJADE_I_0101, intNoOfFilesTransferred);
					rc = true;
					break;
			}
			if (zeroByteCount > 0) {
				state += " " + Messages.getMsg(SOSJADE_I_0102, zeroByteCount);
			}
			logger.debug(state);
			objJadeReportLogger.info(state);
			objOptions.getTextProperties().put(conKeywordSTATE, state);
			return rc;
		}
		catch (Exception e) {
			e.printStackTrace(System.err);
			objJadeReportLogger.info(e.getLocalizedMessage());
			throw new JobSchedulerException(String.format(EXCEPTION_RAISED, e.getLocalizedMessage()));
		}
		finally {
			objOptions.getTextProperties().put(conKeywordSTATE, state);
		}
	}

	/**
	 * @return the state
	 */
	public String getState() {
		String state = (String) objOptions.getTextProperties().get(conKeywordSTATE);
		return state;
	}

	public SOSFileList getFileList() {
		return objSourceFileList;
	}

	/**
	 * 
	 * \brief setJSJobUtilites
	 * 
	 * \details
	 * The JobUtilities are a set of methods used by the SSH-Job or can be used be other, similar, job-
	 * implementations.
	 * 
	 * \return void
	 *
	 * @param pobjJSJobUtilities
	 */
	public void setJSJobUtilites(JSJobUtilities pobjJSJobUtilities) {
		if (pobjJSJobUtilities == null) {
			objJSJobUtilities = this;
		}
		else {
			objJSJobUtilities = pobjJSJobUtilities;
			// objVfs.setJSJobUtilites(pobjJSJobUtilities);
		}
		logger.debug("objJSJobUtilities = " + objJSJobUtilities.getClass().getName());
	}

	/**
	 * 
	 * \brief replaceSchedulerVars
	 * 
	 * \details
	 * Dummy-Method to make sure, that there is always a valid Instance for the JSJobUtilities.
	 * \return 
	 *
	 * @param isWindows
	 * @param pstrString2Modify
	 * @return
	 */
	@Override
	public String replaceSchedulerVars(boolean isWindows, String pstrString2Modify) {
		final String conMethodName = conClassName + "::replaceSchedulerVars";
		logger.debug(conMethodName + " as Dummy-call executed. No Instance of JobUtilites specified.");
		return pstrString2Modify;
	}

	/**
	 * 
	 * \brief setJSParam
	 * 
	 * \details
	 * Dummy-Method to make shure, that there is always a valid Instance for the JSJobUtilities.
	 * \return 
	 *
	 * @param pstrKey
	 * @param pstrValue
	 */
	@Override
	public void setJSParam(String pstrKey, String pstrValue) {
		final String conMethodName = conClassName + "::setJSParam";
		logger.debug(conMethodName + " as Dummy-call executed. No Instance of JobUtilites specified.");
	}

	@Override
	public void setJSParam(String pstrKey, StringBuffer pstrValue) {
		final String conMethodName = conClassName + "::setJSParam";
		logger.debug(conMethodName + " as Dummy-call executed. No Instance of JobUtilites specified.");
	}

	@Override
	public String myReplaceAll(String pstrSourceString, String pstrReplaceWhat, String pstrReplaceWith) {
		String newReplacement = pstrReplaceWith.replaceAll("\\$", "\\\\\\$");
		return pstrSourceString.replaceAll("(?m)" + pstrReplaceWhat, newReplacement);
	}

	@Override
	public String getCurrentNodeName() {
		return "";
	}
}
