/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.dailyschedule.db;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.hibernate.Query;
 

import com.sos.dailyschedule.DailyScheduleFilter;
import com.sos.hibernate.layer.SOSHibernateDBLayer;
import com.sos.scheduler.history.db.SchedulerTaskHistoryDBItem;
import com.sos.scheduler.history.db.SchedulerOrderHistoryDBItem;
 


/**
*  
* \class DailySchedulerDBLayer
* \brief DailySchedulerDBLayer - 
* 
* \details
*
* \section DailySchedulerDBLayer.java_intro_sec Introduction
*
* \section DailySchedulerDBLayer.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author Uwe Risse
* \version 13.09.2011
* \see reference
*
* Created on 13.09.2011 14:40:18
 */

public class DailyScheduleDBLayer  extends SOSHibernateDBLayer{

	@SuppressWarnings("unused")
	private final String	conClassName						= "DailySchedulerDBLayer";
  
	private String			whereFromIso						= null;
	private String			whereToIso							= null;
    private DailyScheduleFilter filter                          = null;
	 

  	 
	
	public DailyScheduleDBLayer(File configurationFile_) {
		super();
		this.setConfigurationFile(configurationFile_);
		this.filter = new DailyScheduleFilter();
		this.filter.setDateFormat("yyyy-MM-dd HH:mm:ss");
		this.filter.setOrderCriteria("schedulePlanned");
	}

	
	 
	public DailyScheduleDBItem getHistory(Long id) {
		return (DailyScheduleDBItem)this.getSession().get(DailyScheduleDBItem.class,id); 
	}
	 
	public int delete() {

		if (session == null) {
			beginTransaction();
		}

		String hql = "delete from DailyScheduleDBItem " + getWhere();

		Query query = session.createQuery(hql);
		
		if (filter.getPlannedFrom() != null && !filter.getPlannedFrom().equals("")) {
			query.setTimestamp("schedulePlannedFrom", filter.getPlannedFrom());
		}
		if (filter.getPlannedTo() != null && !filter.getPlannedTo().equals("")) {
			query.setTimestamp("schedulePlannedTo", filter.getPlannedTo());
		}
		if (filter.getSchedulerId() != null && !filter.getSchedulerId().equals("")) {
			query.setParameter("schedulerId", filter.getSchedulerId());
		}
		int row = query.executeUpdate();

		return row;
	}

	private String getWhere() {
		String where = "";
		String and = "";
 		
		if (filter.getPlannedFrom() != null && !filter.getPlannedFrom().equals("")) {
			where += and + " schedulePlanned>= :schedulePlannedFrom";
			and = " and ";
		}

		if (filter.getPlannedTo() != null && !filter.getPlannedTo().equals("")) {
			where += and + " schedulePlanned <= :schedulePlannedTo ";
			and = " and ";
		}

		if (filter.getSchedulerId() != null && !filter.getSchedulerId().equals("")) {
			where += and + " schedulerId = :schedulerId";
			and = " and ";
		}

		 
		
 		 
		if (where.trim().equals("")) {

		}
		else {
			where = "where " + where;
		}
		return where;

	}

	public List<DailyScheduleDBItem> getDailyScheduleList(int limit) {
		initSession();

 		Query query = session.createQuery("from DailyScheduleDBItem " + getWhere() +  this.filter.getOrderCriteria() + this.filter.getSortMode());

		if (filter.getPlannedFrom() != null && !filter.getPlannedFrom().equals("")) {
			query.setTimestamp("schedulePlannedFrom", filter.getPlannedFrom());
		}
		if (filter.getPlannedTo() != null && !filter.getPlannedTo().equals("")) {
			query.setTimestamp("schedulePlannedTo", filter.getPlannedTo());
		}
		if (filter.getSchedulerId() != null && !filter.getSchedulerId().equals("")) {
			query.setParameter("schedulerId", filter.getSchedulerId());
		}

		if (limit > 0) {
			query.setMaxResults(limit);
		}

		List<DailyScheduleDBItem> daysScheduleList = query.list();
 		return daysScheduleList;

	}

	public List<DailyScheduleDBItem> getWaitingDailyScheduleList(int limit) {
		initSession();

 		Query query = session.createQuery("from DailyScheduleDBItem " + getWhere() + "  and schedulerOrderHistoryId is null  and schedulerHistoryId is null and status = 0  " +  this.filter.getOrderCriteria() + this.filter.getSortMode());

		if (filter.getPlannedFrom() != null && !filter.getPlannedFrom().equals("")) {
			query.setTimestamp("schedulePlannedFrom", filter.getPlannedFrom());
		}
		if (filter.getPlannedTo() != null && !filter.getPlannedTo().equals("")) {
			query.setTimestamp("schedulePlannedTo", filter.getPlannedTo());
		}
		if (filter.getSchedulerId() != null && !filter.getSchedulerId().equals("")) {
			query.setParameter("schedulerId", filter.getSchedulerId());
		}

		if (limit > 0) {
			query.setMaxResults(limit);
		}

		List<DailyScheduleDBItem> daysScheduleList = query.list();
 		return daysScheduleList;

	}
	
	


	public DailyScheduleFilter getFilter() {
		return filter;
	}

	public void setWhereFrom(Date whereFrom) {
		this.filter.setPlannedFrom(whereFrom);
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm");
		whereFromIso = formatter.format(whereFrom);
	}

	public void setWhereTo(Date whereTo) {
		this.filter.setPlannedTo(whereTo);
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm");
		whereToIso = formatter.format(whereTo);
	}

	public void setWhereFrom(String whereFrom) throws ParseException {
		if (whereFrom.equals("")) {
			this.filter.setPlannedFrom("");
		}
		else {
			SimpleDateFormat formatter = new SimpleDateFormat(filter.getDateFormat());
			Date d = formatter.parse(whereFrom);
			setWhereFrom(d);
		}
	}

	public void setWhereTo(String whereTo) throws ParseException {
		if (whereTo.equals("")) {
			this.filter.setPlannedTo("");
		}
		else {
			SimpleDateFormat formatter = new SimpleDateFormat(filter.getDateFormat());
			Date d = formatter.parse(whereTo);
			setWhereTo(d);
		}
	}
	

	public Date getWhereFrom() {
		return filter.getPlannedFrom();
	}

	public Date getWhereTo() {
		return filter.getPlannedTo();
	}

	public void setWhereSchedulerId(String whereSchedulerId) {
		this.filter.setSchedulerId(whereSchedulerId);
	}

	public void setDateFormat(String dateFormat) {
		this.filter.setDateFormat(dateFormat);
	}


	public String getWhereFromIso() {
		return whereFromIso;
	}


	public String getWhereToIso() {
		return whereToIso;
	}
	
	public void setFilter(DailyScheduleFilter filter) {
		this.filter = filter;
	}
	
	
	public boolean contains(SchedulerTaskHistoryDBItem schedulerHistoryDBItem) {
		if (session == null) {
			initSession();
			transaction = session.beginTransaction();
		}
		Query query = session.createQuery("from DailyScheduleDBItem where schedulerId=:schedulerId and schedulerHistoryId=:schedulerHistoryId");
		query.setParameter("schedulerId", schedulerHistoryDBItem.getSpoolerId());
		query.setParameter("schedulerHistoryId", schedulerHistoryDBItem.getId());

	 	List<DailyScheduleDBItem> daysScheduleList = query.list();
		return daysScheduleList.size() > 0;
	}

	public boolean contains(SchedulerOrderHistoryDBItem schedulerOrderHistoryDBItem) {
		if (session == null) {
			initSession();
			transaction = session.beginTransaction();
		}
		Query query = session.createQuery("from DailyScheduleDBItem where schedulerId=:schedulerId and jobChain=:jobChain and schedulerOrderHistoryId=:schedulerOrderHistoryId");
		query.setParameter("schedulerId", schedulerOrderHistoryDBItem.getSpoolerId());
		query.setParameter("schedulerOrderHistoryId", schedulerOrderHistoryDBItem.getHistoryId());
		query.setParameter("jobChain", schedulerOrderHistoryDBItem.getJobChain());
        List<DailyScheduleDBItem> daysScheduleList = query.list();
 		return daysScheduleList.size() > 0;
	}



}
