/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.creator;

import java.util.Iterator;
import java.util.List;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Basics.JSToolBox;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.classes.JobNetException;
import com.sos.localization.Messages;

public class JobNetBootstrapOrder extends JobNetOrder {
	
	private final static Logger logger = Logger.getLogger(JobNetBootstrapOrder.class);
	
	private static final Messages msg = new JSToolBox(JobNetConstants.strBundleBaseName).getMessageObject();
	private final static String JOBNETC_I_0004 = "JOBNETC_I_0004"; // %1$s is successor of %2$s

	private static final String JOBNETC_E_0001 = "JOBNETC_E_0001"; 		// %1$s is successor of %2$s but %3$s is not predecessor of %4$s
	private final static String JOBNETC_E_0002 = "JOBNETC_E_0002"; 		// %1$s is predecessor of %2$s but %3$s is not successor of %4$s.
	
	@SuppressWarnings("unused")
	private static final String conSVNVersion = "$Id: JobNetBootstrapOrder.java 17092 2012-05-10 10:08:53Z ss $";

	private static JobNetNodeList orderNodes; 

	public JobNetBootstrapOrder(ObjectModelConnector connection, String baseFolder, String clusterId, String orderId, String jobChain, int maxNodes) {
		super(connection, baseFolder, clusterId, getOrderId(orderId,jobChain), getJobChain(orderId,jobChain), initNodeList(maxNodes));
		this.setBootstrap(true);
	}
	
	private static JobNetNodeList initNodeList(int maxNodes) {
		orderNodes = new JobNetNodeList(maxNodes);
		return orderNodes;
	}
	
	/**
	 * \brief gets the orderId with live folder path
	 *
	 * @param orderId
	 * @param jobChain
	 * @return
	 */
	private static String getOrderId(String orderId, String jobChain) {
		String result = orderId;
		if (jobChain.contains("/")) {
			String[] arr = jobChain.split("/");
			result = "/";
			for (int i = 0; i < arr.length-1; i++) {
				result += arr[i] + "/";
			}
			result += orderId;
		}
		return result;
	}
	
	/**
	 * \brief gets the jobChain name without live folder path
	 *
	 * @param orderId
	 * @param jobChain
	 * @return
	 */
	private static String getJobChain(String orderId, String jobChain) {
		String result = jobChain;
		if (jobChain.contains("/")) {
			String[] arr = jobChain.split("/");
			result = arr [arr.length-1];
		}
		return result;
	}
	
	public void createJobNet() {
		iterateJobNet(this,1);
	}
	
	private void iterateJobNet(JobNetOrder order, int level) {
		orderNodes.addOrderToNodesCollection(order);
		if (order.hasSuccessors()) {
			List<JobNetOrder> successors = order.getSuccessors();
			Iterator<JobNetOrder> it = successors.iterator();
			while(it.hasNext()) {
				JobNetOrder s = it.next();
				if(!orderNodes.containsKey(s.getJobNetNodeName())) {
					logger.debug(msg.getMsg(JOBNETC_I_0004,s.getJobNetNodeName(),order.getJobNetNodeName()));
					logger.debug("level=" + level);
					iterateJobNet(s, level+1);
				}
			}
		}
	}
	
	public JobNetNodeList getJobNet() {
		return orderNodes;
	}
	
	public void validateJobNet() {
		for(JobNetOrder order : getJobNet().values()) {
			for(JobNetOrder successor : order.getSuccessors()) {
				if (!successor.hasPredecessor(order)) {
					String msgText = msg.getMsg(JOBNETC_E_0001,successor.getJobNetNodeName(),order.getJobNetNodeName(),order.getJobNetNodeName(),successor.getJobNetNodeName());
					logger.error(msgText);
					throw new JobNetException(msgText);
				}
			}
			for(JobNetOrder predecessor : order.getPredecessors()) {
				if (!predecessor.hasSuccessor(order)) {
					String msgText = msg.getMsg(JOBNETC_E_0002,predecessor.getJobNetNodeName(),order.getJobNetNodeName(),order.getJobNetNodeName(),predecessor.getJobNetNodeName());
					logger.error(msgText);
					throw new JobNetException(msgText);
				}
			}
		}
	}

}
