/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.dispatcher;
import java.io.File;
import java.util.HashMap;
import java.util.List;

import org.apache.log4j.Logger;

import com.sos.JSHelper.io.Files.JSFile;
import com.sos.jobnet.classes.EdgeType;
import com.sos.jobnet.classes.JobNetCollection;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.classes.Status;
import com.sos.jobnet.classes.Status.enuStatus;
import com.sos.jobnet.db.JobNetEdgesDBItem;
import com.sos.jobnet.db.JobNetEdgesDBLayer;
import com.sos.jobnet.db.JobNetNodeDBItem;
import com.sos.jobnet.db.JobNetNodeDBLayer;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.db.JobNetPlanDBLayer;
import com.sos.jobnet.db.JobNetPlanFilter;
import com.sos.jobnet.interfaces.IJobNetCollection;
import com.sos.jobnet.interfaces.IDispatchHandler;
import com.sos.jobnet.interfaces.IPlanItem;
import com.sos.jobnet.options.JobNetOptions;
import com.sos.i18n.I18NBase;
import com.sos.i18n.annotation.I18NResourceBundle;

/**
* \class JobNetDBDispatcher 
* 
* \brief JobNetDBDispatcher - 
* 
* \details
*
* \section JobNetDBDispatcher.java_intro_sec Introduction
*
* \section JobNetDBDispatcher.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id: JobNetDBDispatcher.java 16693 2012-03-02 10:25:19Z ur $
* \see reference
*
* Created on 22.02.2012 13:56:28
 */
/**
 * @author oh
 *
 */
public class JobNetDBDispatcher  extends I18NBase  implements IDispatchHandler {
	@SuppressWarnings("unused")

	private final String	conClassName	= "JobNetDBDispatcher";
	public  final String	conSVNVersion	= "$Id: JobNetDBDispatcher.java 16693 2012-03-02 10:25:19Z ur $";
	
 	private static final Logger	logger					= Logger.getLogger(JobNetDBDispatcher.class);
	private JobNetOptions		objOptions				= null;
	private File				configurationFile		= null;
	private JobNetPlanDBItem	curNode					= null;
	private JobNetPlanDBLayer	jobNetPlanDBLayer		= null;
	JobNetNodeDBLayer jobNetNodeDBLayer					= null;
	private JobNetEdgesDBLayer	jobNetEdgesDBLayer		= null;
	private IJobNetCollection 	dispatchNodes 		= new JobNetCollection();
	private boolean			 	curNodeFound	 		= true;
	
	private static final String	JOBNETDBW_E_0010		= "JOBNETDBW_E_0010";		// %s: No item found for the current node (%s) in database table SCHEDULER_JOB_NET_PLAN.
 	

 
	
	public JobNetDBDispatcher(JobNetOptions jobNetOptions) {
		super(JobNetConstants.strBundleBaseName);
		this.objOptions = jobNetOptions;
		doInit();
	}
	
	public JobNetDBDispatcher(HashMap<String, String> jobNetOptions) {
		super(JobNetConstants.strBundleBaseName);
		setOptionsUsingHashMap(jobNetOptions);
		doInit();
	}

	// public void doInit
	public void doInit() {
		@SuppressWarnings("unused")
		final String	conMethodName	= conClassName + "::doInit";
		
		this.configurationFile 	= new File(Options().hibernate_connection_config_file.Value());
	    this.jobNetPlanDBLayer 	= new JobNetPlanDBLayer(configurationFile, Options().scheduler_id.Value());
		this.jobNetEdgesDBLayer = new JobNetEdgesDBLayer(configurationFile);
		this.jobNetNodeDBLayer = new JobNetNodeDBLayer(configurationFile, Options().scheduler_id.Value());

	} 
	
	
	public JobNetOptions Options() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Options";
		if (objOptions == null) {
			objOptions = new JobNetOptions();
		}
		return objOptions;
	} // public JobNetOptions Options

	
	public void setOptionsUsingHashMap(HashMap<String, String> objHshMap) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setOptionsUsingHashMap";
		try {
			Options().setAllOptions(objHshMap);
		}
		catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	} // public void setOptionsUsingHashMap
	
	
	/**
	 * \brief getCurNode
	 * 
	 * \details
	 */
	private JobNetPlanDBItem getCurNode() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getCurNode";
		if (curNode == null  && curNodeFound) {
			
			String uuidJobnetIdentifier 			= Options().uuid_jobnet_identifier.Value();
			String jobNet 							= Options().jobnet.Value();
			String node 							= Options().node.Value();
			String schedulerId 						= Options().scheduler_id.Value();
			JSFile hibernateConnectionConfigFile 	= Options().hibernate_connection_config_file.JSFile();
			String curNodeTitle 					= uuidJobnetIdentifier+"/"+schedulerId+":"+jobNet+":"+node;

			
			JobNetPlanFilter jobNetPlanFilter = new JobNetPlanFilter();
			jobNetPlanFilter.setSchedulerId(schedulerId);
			jobNetPlanFilter.setUuid(uuidJobnetIdentifier);
			jobNetPlanFilter.setJobnet(jobNet);
			jobNetPlanFilter.setNode(node);
 			jobNetPlanDBLayer.setFilter(jobNetPlanFilter);
 			
 			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "uuid_jobnet_identifier", uuidJobnetIdentifier));
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "jobnet", jobNet));
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "node", node));
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "scheduler_id", schedulerId));
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "hibernate_connection_config_file", hibernateConnectionConfigFile.getAbsolutePath()));
			
				 
			List<JobNetPlanDBItem> jobNetPlanDBItem  = jobNetPlanDBLayer.getJobnetPlanListWithLock(0);
 			
 			if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
				curNodeFound = false;
				String msg = Messages.getMsg(JOBNETDBW_E_0010, conMethodName, curNodeTitle);
				logger.error(msg);
				throw new JobNetException(msg);
			}
			else {
				curNode = jobNetPlanDBItem.get(0);
				logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0010, conMethodName, curNode.getTitle()));
			}
		}
		return curNode;
	} // public void readCurNode

	
	/**
	 * \brief updateDispatcherStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateDispatcherStatus(enuStatus status) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::updateDispatcherStatus";
 		getCurNode().setStatusDispatcher(status.value());
		update();
 	} // private void updateDispatcherStatus
	
	
	/**
	 * \brief updateRunnerStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateRunnerStatus(enuStatus status) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::updateRunnerStatus";
 		getCurNode().setStatusRunner(status.value());
		update();
 	} // private void updateRunnerStatus
	
	
	/**
	 * \brief updateNodeStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateNodeStatus(enuStatus status) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::updateNodeStatus";
		getCurNode().setStatusNode(status.value());
		update();
 	} // private void updateNodeStatus

	
	 
	/**
	 * \brief update
	 * 
	 * \details
	 */
	private void update() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::update";
		jobNetPlanDBLayer.beginTransaction();
		jobNetPlanDBLayer.saveOrUpdate(curNode);
		jobNetPlanDBLayer.commit();
	} // private void update

	
	/**
	 * \brief setDispatcherStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setDispatcherStatusToRunning() {
		updateDispatcherStatus(enuStatus.RUNNING);
	}

	/**
	 * \brief setNodeStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToFinished() {
		updateNodeStatus(enuStatus.FINISHED);
	}
	


	/**
	 * \brief isDispatcherSkipped
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isDispatcherSkipped() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isDispatcherSkipped";
		return getCurNode().getIsDispatcherSkipped();
	}
	

	 
	/**
	 * \brief setNodeStatusToDispatching
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToDispatching() {
		updateNodeStatus(enuStatus.DISPATCHING);
	}

	/**
	 * \brief setRunnerStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setRunnerStatusToFinished() {
		updateRunnerStatus(enuStatus.FINISHED);
	}

	/**
	 * \brief setDispatcherStatusToFinish
	 * 
	 * \details
	 */
	@Override
	public void setDispatcherStatusToFinished() {
		updateDispatcherStatus(enuStatus.FINISHED);
	}

	/**
	 * \brief setDispatcherStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setDispatcherStatusToError() {
		updateDispatcherStatus(enuStatus.ERROR);
	}

	/**
	 * \brief setNodeStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToError() {
		updateNodeStatus(enuStatus.ERROR);
	}

	
	@Override
	public void setRunnerStatusToError() {
		updateRunnerStatus(enuStatus.ERROR);
		
	}

	@Override
	public Boolean isRunnerSkipped() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isRunnerSkipped";
		return getCurNode().getIsRunnerSkipped();	
	}
	
	
	@Override
	public int getNumberOfDispatchingNodes() {
		if(dispatchNodes.size() < 0) {
			getDispatchingNodes();
		}
		return dispatchNodes.size();
	}

	@Override
	public IJobNetCollection getDispatchingNodes() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getDispatchforNodes";
		getCurNode();
		
		String uuidJobnetIdentifier 			= Options().uuid_jobnet_identifier.Value();
		String jobNet 							= Options().jobnet.Value();
 		String schedulerId 						= Options().scheduler_id.Value();
 
 		
		List<JobNetEdgesDBItem> jobnetEdgesList = jobNetEdgesDBLayer.getSuccessors(curNode.getJobnetNodeDBItem()); 
 		
		int status = -1;
		Status objStatus = new Status();
		for (JobNetEdgesDBItem jobnetEdgesItem : jobnetEdgesList) {
			
			JobNetNodeDBItem jobNetNodeDBItem = jobNetNodeDBLayer.getNode(jobnetEdgesItem.getChildNodeId());
			Long successor = jobNetNodeDBItem.getNodeId();
			
			JobNetPlanFilter jobNetPlanFilter = new JobNetPlanFilter();
			jobNetPlanDBLayer.setFilter(jobNetPlanFilter);
			
			jobNetPlanDBLayer.getFilter().setNodeId(successor);
			jobNetPlanFilter.setSchedulerId(schedulerId);
			jobNetPlanFilter.setUuid(uuidJobnetIdentifier);
			jobNetPlanFilter.setJobnet(jobNet);
			
			JobNetPlanDBItem jobnetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanList(0).get(0);
			status = jobnetPlanDBItem.getStatusNode();
			logger.info(String.format("The node %1s has the status %2s", successor, status));
			if (objStatus.isNotProcessed(status)) {
				dispatchNodes.add(jobnetPlanDBItem);
			}else{
				logger.info(String.format("The node %1s will NOT be startet. Already running", successor));
			}
		}
		return dispatchNodes;
	}

	

	@Override
	public void open() {
		logger.debug(String.format("....Class: %s1    conSVN: %s2",conClassName, conSVNVersion));
	    logger.debug(String.format("....Scheduler/Cluster: %s1",Options().scheduler_id.Value()));
		logger.debug(String.format("....Using hibernate configuration file: %1s",Options().hibernate_connection_config_file.Value()));
	    logger.debug(String.format("....Dispatching node: %s1 in jobnet %s2",Options().node.Value(),Options().jobnet.Value()));
 	}

	@Override
	public void close() {
		logger.debug(String.format("\n..........Dispatching finished"));
	}


 

}
