/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.waiter;
import java.util.HashMap;

import org.apache.log4j.Logger;

import com.sos.i18n.I18NBase;
import com.sos.i18n.annotation.I18NResourceBundle;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.classes.Status;
import com.sos.jobnet.dispatcher.JobNetDispatcher;
import com.sos.jobnet.interfaces.IJobNetCallback;
import com.sos.jobnet.interfaces.IJobNetCollection;
import com.sos.jobnet.interfaces.IPlanItem;
import com.sos.jobnet.interfaces.IWaitHandler;
import com.sos.jobnet.options.JobNetOptions;

/**
* \class JobNetWaiter 
* 
* \brief JobNetWaiter - 
* 
* \details
*
* \section JobNetWaiter.java_intro_sec Introduction
*
* \section JobNetWaiter.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id: JobNetWaiter.java 16929 2012-04-03 09:03:12Z oh $
* \see reference
*
* Created on 24.02.2012 13:02:19
 */
/**
 * @author oh
 *
 */
@I18NResourceBundle(baseName = "com_sos_jobnet_messages", defaultLocale = "en")
public class JobNetWaiter extends I18NBase implements IJobNetCallback {

	private final String		conClassName	= "JobNetWaiter";
	@SuppressWarnings("unused")
	private	final String		conSVNVersion	= "$Id: JobNetWaiter.java 16929 2012-04-03 09:03:12Z oh $";
	private static final Logger	logger			= Logger.getLogger(JobNetWaiter.class);
	private IWaitHandler		objWaitHandler	= null;
	private JobNetOptions		objOptions		= null;
	private IJobNetCallback		objJNCallback 	= this;
	
	private static final String	JOBNETW_I_0001		= "JOBNETW_I_0001";			// %s: Check %d of %d
	private static final String	JOBNETW_I_0002		= "JOBNETW_I_0002";			// %s: Check %d
	private static final String	JOBNETW_I_0003		= "JOBNETW_I_0003";			// %s: %s is called because the %s is skipped
	private static final String	JOBNETW_I_0005		= "JOBNETW_I_0005";			// %s: Max. checks reached
	private static final String	JOBNETW_T_0001		= "JOBNETW_T_0001";			// Check(%d|%d): %s
	private static final String	JOBNETW_T_0002		= "JOBNETW_T_0002";			// Start check for waiting nodes
	private static final String	JOBNETW_T_0003		= "JOBNETW_T_0003";			// DISPATCHER is started, RUNNER is skipped
	private static final String	JOBNETW_T_0004		= "JOBNETW_T_0004";			// RUNNER is started
	private static final String	JOBNETW_T_0005		= "JOBNETW_T_0005";			// RUNNER is 'On Demand'!
	private static final String	JOBNETW_T_0006		= "JOBNETW_T_0006";			// Waiting for
	
	public JobNetWaiter() {
		super(JobNetConstants.strBundleBaseName);
	}

	public JobNetOptions Options() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Options";
		if (objOptions == null) {
			objOptions = new JobNetOptions();
		}
		return objOptions;
	} // public JobNetOptions Options

	@SuppressWarnings("hiding")
	public void setOptions(JobNetOptions objOptions) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setOptions";
		this.objOptions = objOptions;
	} // public void setOptions

	public void setOptionsUsingHashMap(HashMap<String, String> objHshMap) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setOptionsUsingHashMap";
		try {
			Options().setAllOptions(objHshMap);
		}
		catch (Exception e) {
			throw new JobNetException(e);
		}
	} // public void setOptionsUsingHashMap

	public void doInit() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::doInit";
		objWaitHandler = new JobNetDBWaiter(Options());
	} // public void doInit
	
	
	public void setIJobNetCallback(IJobNetCallback pobjJobNetCallback) {
		if (objJNCallback == null) {
			objJNCallback = this;
		}
		else {
			objJNCallback = pobjJobNetCallback;
		}
	}

	
	public boolean run() {
		final String conMethodName 	= conClassName + "::run";
		boolean ok 					= true;
		int intNoOfChecks 			= 1;
		int intNoOfWaitingNodes		= 0;
		boolean bMaxChecksReached 	= false;
		boolean bRunnerIsOnDemand 	= false;
		IJobNetCollection jobNetCollection = null;
		int intMaxNumberOfChecks 	= Options().max_number_of_checks.value();
		int intCheckInterval 		= Options().check_interval.getTimeAsSeconds();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "max_number_of_checks", intMaxNumberOfChecks));
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "check_interval", intCheckInterval));
		
		try {
			objJNCallback.setStateText(Messages.getMsg(JOBNETW_T_0002));
			objWaitHandler.open();
			objWaitHandler.setWaiterStatusToRunning();
			objWaitHandler.setNodeStatusToWaiting();
			do {
				logTrial(conMethodName, intNoOfChecks, intMaxNumberOfChecks);
				
				if (objWaitHandler.isRunnerSkipped()) {
					bRunnerIsOnDemand 	= false;
				}
				else {
					bRunnerIsOnDemand 	= objWaitHandler.isRunnerOnDemand();
				}
				if (objWaitHandler.isWaiterSkipped() || objWaitHandler.isBootstrap()) {
					intNoOfWaitingNodes = 0;
				}
				else {
					jobNetCollection 	= objWaitHandler.getWaitingNodes();
					intNoOfWaitingNodes = jobNetCollection.size();
				}
				if (intNoOfWaitingNodes > 0 || bRunnerIsOnDemand) {
					objJNCallback.setStateText(getStateText(jobNetCollection, bRunnerIsOnDemand, intNoOfChecks, intMaxNumberOfChecks));
					if (intMaxNumberOfChecks > 0) {
						if (intMaxNumberOfChecks <= intNoOfChecks) {
							bMaxChecksReached = true;
							logger.info(Messages.getMsg(JOBNETW_I_0005, conMethodName));
							break;
						}
					}
					Thread.sleep(intCheckInterval*1000);
					intNoOfChecks++;
					objWaitHandler.refresh();
				}
			}
			while (intNoOfWaitingNodes > 0 || bRunnerIsOnDemand);
			
			if (bMaxChecksReached) {
				ok = false;
				objWaitHandler.setWaiterStatusToError();
				objWaitHandler.setNodeStatusToError();
				Thread.sleep(5*1000); //Damit der letzte StateText in JOC auch zu lesen ist
			}
			else {
				if (objWaitHandler.isRunnerSkipped()) {
					objWaitHandler.setWaiterStatusToFinish();
					objWaitHandler.setNodeStatusToRunning();
					objJNCallback.skipRunner();
					objJNCallback.setStateText(Messages.getMsg(JOBNETW_T_0003));
				}
				else {
					objWaitHandler.setWaiterStatusToFinish();
					objWaitHandler.setRunnerStatusToRunning();
					objWaitHandler.setStartTime();
					objWaitHandler.setNodeStatusToRunning();
					objJNCallback.setStateText(Messages.getMsg(JOBNETW_T_0004));
				}
			}
		}
		catch (Exception e) {
			ok = false;
			try {
				objWaitHandler.setWaiterStatusToError();
				objWaitHandler.setNodeStatusToError();
			}
			catch (Exception e1) {
				e1.printStackTrace(System.err);
//				throw new JobNetException("Exception raised", e1);
			}
			e.printStackTrace(System.err);
			throw new JobNetException("Exception raised", e);
		}
		finally {
//			objJNCallback.setStateText("");
			objWaitHandler.close();
		}
		return ok;
	} // public JobNetWaiter run
	
	
	
	private void logTrial(String conMethodName, int intNoOfChecks, int intMaxNumberOfChecks) {
		if (intMaxNumberOfChecks > 0) {
			logger.info(Messages.getMsg(JOBNETW_I_0001, conMethodName, intNoOfChecks, intMaxNumberOfChecks));
		}
		else {
			logger.info(Messages.getMsg(JOBNETW_I_0002, conMethodName, intNoOfChecks));
		}
	}
	
	
	private String getStateText(IJobNetCollection jobNetCollection, boolean bRunnerIsOnDemand, int intNoOfChecks, int intMaxNumberOfChecks) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setStateText";
		String msg = "";
		String trials = "";
		Status status = new Status();
		if (bRunnerIsOnDemand) {
			msg += " ";
			msg += Messages.getMsg(JOBNETW_T_0005);
		}
		if (jobNetCollection != null) {
			if (jobNetCollection.size() > 0) {
				msg += " ";
				msg += Messages.getMsg(JOBNETW_T_0006);
				msg += " ";
			}
			for (int j = 0; j < jobNetCollection.size(); j++) {
				IPlanItem node = jobNetCollection.get(j);
				if (j>0) {
					msg += ", ";
				}
				msg += String.format("%s(%s)", node.getJobnetNodeItem().getNode(), status.getMsg(node.getStatusNode()));
			}
		}
		
		if (intMaxNumberOfChecks > -1) {
			trials = String.format("(%d|%d)", intNoOfChecks, intMaxNumberOfChecks);
		}
		else {
			trials = String.format(" %d", intNoOfChecks);
		}
		msg = Messages.getMsg(JOBNETW_T_0001, trials, msg);
		logger.info(msg);
		return msg;
	}

	@Override
	public int getExitCode() {
		return 0;
	}

	@Override
	public void startNode(IPlanItem planNode) {
	}

	@Override
	public void setStateText(String stateText) {
		logger.info(stateText);
	}

	@Override
	public void skipRunner() {
		final String conMethodName = conClassName + "::skipRunner";
		logger.info(Messages.getMsg(JOBNETW_I_0003, conMethodName));
		JobNetDispatcher dispatcher = new JobNetDispatcher();
		dispatcher.setOptions(Options());
		dispatcher.doInit();
		dispatcher.run();
	} // public void skipRunner
	
	
	
}
