/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.history;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

 import com.sos.hibernate.classes.DbItem;
import com.sos.hibernate.classes.SOSHibernateFilter;
import com.sos.scheduler.history.classes.HistorySeverity;
import com.sos.scheduler.history.classes.SOSIgnoreList;
import com.sos.scheduler.history.db.SchedulerOrderHistoryDBItem;

/**
* \class SchedulerOrderHistoryFilter 
* 
* \brief SchedulerOrderHistoryFilter - 
* 
* \details
*
* \section SchedulerOrderHistoryFilter.java_intro_sec Introduction
*
* \section SchedulerOrderHistoryFilter.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author Uwe Risse
* \version 14.12.2011
* \see reference
*
* Created on 14.12.2011 13:53:37
 */

public class SchedulerHistoryFilter extends SOSHibernateFilter implements com.sos.hibernate.interfaces.ISOSHibernateFilter {

	@SuppressWarnings("unused")
	private final String	conClassName	= "SchedulerHistoryFilter";
	private String			dateFormat		= "yyyy-MM-dd HH:mm:ss";
	private Date			executedFrom;
	private Date			executedTo;
	private Date			startTime;
	private Date			endTime;
	private String			schedulerId		= "";

	private String			executedFromIso	= null;
	private String			executedToIso	= null;
	private SOSIgnoreList	ignoreList=null;
	
	private boolean onlyJobs=false;
	private boolean onlyJobChains=false;


	public SchedulerHistoryFilter() {
		ignoreList = new SOSIgnoreList();
	}

	public boolean isOnlyJobs() {
		return onlyJobs;
	}

	public void setOnlyJobs(boolean onlyJobs) {
		this.onlyJobs = onlyJobs;
	}

	public boolean isOnlyJobChains() {
		return onlyJobChains;
	}

	public void setOnlyJobChains(boolean onlyJobChains) {
		this.onlyJobChains = onlyJobChains;
	}

	public String getExecutedFromIso() {
		return executedFromIso;
	}

	public void setExecutedFromIso(String executedFromIso) {
		this.executedFromIso = executedFromIso;
	}

	public String getExecutedToIso() {
		return executedToIso;
	}

	public void setExecutedToIso(String executedToIso) {
		this.executedToIso = executedToIso;
	}
 	

	public SOSIgnoreList getIgnoreList() {
		return ignoreList;
	}

	public String getDateFormat() {
		return dateFormat;
	}

	public void setDateFormat(String dateFormat) {
		this.dateFormat = dateFormat;
	}

	public Date getExecutedFrom() {
		return executedFrom;
	}

	public void setExecutedFrom(String executedFrom) throws ParseException {
		if (executedFrom.equals("")) {
			this.executedFrom = null;
		}
		else {
			SimpleDateFormat formatter = new SimpleDateFormat(dateFormat);
			Date d = formatter.parse(executedFrom);
			setExecutedFrom(d);
		}
	}

	public Date getExecutedTo() {
		return executedTo;
	}

	public void setExecutedTo(String executedTo) throws ParseException {
		if (executedTo.equals("")) {
			this.executedTo = null;
		}
		else {
			SimpleDateFormat formatter = new SimpleDateFormat(dateFormat);
			Date d = formatter.parse(executedTo);
			setExecutedTo(d);
		}
	}

	 
	public String getSchedulerId() {
		return schedulerId;
	}

	public void setSchedulerId(String schedulerId) {
		this.schedulerId = schedulerId;
	}

	public void setExecutedFrom(Date from) {
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd 00:00:00");
	    String d = formatter.format(from);
		try {
			formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			this.executedFrom = formatter.parse(d);
		}
		catch (ParseException e) {
 			e.printStackTrace();
		}	
		
 		formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		executedFromIso = formatter.format(from);	
	}

	public void setExecutedTo(Date to) {
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd 23:59:59");
	    String d = formatter.format(to);
		try {
			formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
			this.executedTo = formatter.parse(d);
		}
		catch (ParseException e) {
 			e.printStackTrace();
		}	
		
 		formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		executedToIso = formatter.format(to);
	}

	public void setStartTime(Date start) {
		this.startTime = start;
 	}	
	
	public void setEndTime(Date end) {
		this.endTime = end;
 	}
	

	public Date getStartTime() {
		return startTime;
	}

	public Date getEndTime() {
		return endTime;
	}
	
	@Override
	public String getTitle() {
	    String ignoreList = " ";
        int ignoreOrderCount = getIgnoreList().size();
        int ignoreJobCount =  getIgnoreList().size();
        if (ignoreOrderCount > 0 || ignoreJobCount > 0){
            ignoreList = String.format("%1s Jobs %2s Orders ignored",ignoreJobCount,ignoreOrderCount);
        }
		
		String s = "";
		if (schedulerId != null && !schedulerId.equals("")) {
			   s += String.format("Id: %s ",schedulerId);
		}
		if (executedFrom != null) {
			   s += String.format("from: %s " ,date2Iso(executedFrom));
		}

		if (executedTo != null) {
			   s += String.format("to: %s " ,date2Iso(executedTo));
		}
		
        String title =  String.format("%1s %2s", s,ignoreList);
		return title;
	}

	@Override
	public boolean isFiltered(DbItem h) {
			return false;
	}

	
	

}
