/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// zschimmer.h                                      2000 Joacim Zschimmer
// $Id: odbc.h 11394 2005-04-03 08:30:29Z jz $

#ifndef __ZSCHIMMER_ODBC_H
#define __ZSCHIMMER_ODBC_H

#include <windows.h>
#include <sqlext.h>
#include <sql.h>
#include "zschimmer.h"

namespace zschimmer {
namespace odbc {

//-------------------------------------------------------------------------------------------------

void check_return( SQLRETURN );

//--------------------------------------------------------------------------------------Odbc_handle

struct Odbc_handle : Object
{
                                Odbc_handle             ( SQLSMALLINT type, SQLHANDLE outer_handle );
    virtual                    ~Odbc_handle             ();

    virtual                     close                   ();
    void                        check                   ( SQLRETURN );
    

    SQLSMALLINT                _type;
    SQLHANDLE                  _handle;
};

//----------------------------------------------------------------------------------------------Env

struct Env : Odbc_handle
{
                                Env                     ()                                          : Odbc_handle( SQL_HANDLE_ENV, SQL_NULL_HANDLE ) {}

                                operator SQLHENV        ()                                          { return (SQLHENV)_handle; }
};

//---------------------------------------------------------------------------------------------Conn

struct Conn : Odbc_handle
{
                                Conn                    ( Env* env )                                : Odbc_handle( SQL_HANDLE_DBC, *env ), _env(env) {}

    void                        DriverConnect           ( SQLHWND, const string& conn, string* conn_ret = NULL, SQLUSMALLINT DriverCompletion = SQL_DRIVER_NOPROMPT );
    void                        connect_to_file         ( const string& filename );

                                operator SQLHDBC        ()                                          { return (SQLHDBC)_handle; }

    ptr<Env>                   _env;
};

//---------------------------------------------------------------------------------------------Stmt

struct Stmt : Odbc_handle
{
                                Stmt                    ( Conn* conn )                              : Odbc_handle( SQL_HANDLE_DBC, *conn ), _conn(conn) {}
                               ~Stmt                    ();

                                operator SQLHDBC        ()                                          { return (SQLHDBC)_handle; }

    void                        Prepare                 ( const string& stmt );
    void                        BindCol                 ( SQLUSMALLINT ColumnNumber, SQLSMALLINT TargetType, 
                                                          SQLPOINTER TargetValuePtr, SQLINTEGER BufferLength, SQLLEN* StrLen_or_Ind = NULL );
    void                        Execute                 ();
    bool                        Fetch                   ();                                         // false: EOF

    
    ptr<Conn>                  _conn;
};

//-------------------------------------------------------------------------------------------------

} //namespace odbc
} //namespace zschimmer


#endif
